from biogeme import *
from headers import *
from loglikelihood import *
from statistics import *
from nested import *
#Parameters to be estimated
# Arguments:
#   - 1  Name for report; Typically, the same as the variable.
#   - 2  Starting value.
#   - 3  Lower bound.
#   - 4  Upper bound.
#   - 5  0: estimate the parameter, 1: keep it fixed.
#
ASC_CAR = Beta('ASC_CAR',3.53,-10,80,0,'Car cte.')
ASC_TRAIN = Beta('ASC_TRAIN',2.01,-10,80,0,'Train cte.')
ASC_BUS = Beta('ASC_BUS',0,-10,10,1,'Bus cte.')
B_TIME = Beta('B_TIME',-2.80,-40,10,0,'Travel time')
B_TIME_CAR = Beta('B_TIME_CAR',0,-40,10,0,'Travel time car')
B_COST = Beta('B_COST',0,-10,10,0,'Travel cost')
B_FEMALE = Beta('B_FEMALE',0,-100,100,0,'FEMALE FEMALE')
B_WORKING = Beta('B_WORKING',1.40,-10,10,0,'WORK COMMUTE PURPOSE')
B_AGE50OVER = Beta('B_AGE50OVER',3.91,-20,80,0,'AGE50')
B_AGE30UNDER = Beta('B_AGE30UNDER',0,-20,10,0,'AGE30')
B_EGRESS_DIS_TRAIN=Beta('B_EGRESS_DIS_TRAIN',-1.23,-10,10,0,'EGRESS_DIS_TRAIN')
B_EGRESS_DIS_ACCESS=Beta('B_EGRESS_DIS_ACCESS',-1.23,-10,10,0,'EGRESS_ACC_TRAIN')
MU_ROAD = Beta('MU_ROAD',1.0,-10,10,0)

# Utility functions

#If the person has a GA (season ticket) her incremental cost is actually 0 
#rather than the cost value gathered from the
# network data. 



# For numerical reasons, it is good practice to scale the data to
# that the values of the parameters are around 1.0. 
# A previous estimation with the unscaled data has generated
# parameters around -0.01 for both cost and time. Therefore, time and
# cost are multipled my 0.01.

# The following statements are designed to preprocess the data. It is
# like creating a new columns in the data file. This should be
# preferred to the statement like
# TRAIN_TT_SCALED = TRAIN_TT / 100.0
# which will cause the division to be reevaluated again and again,
# throuh the iterations. For models taking a long time to estimate, it
# may make a significant difference.
 
TRAIN_TT_SCALED = DefineVariable('TRAIN_TT_SCALED', TRAIN_TIME / 100.0)
BUS_TT_SCALED = DefineVariable('BUS_TT_SCALED', BUS_TIME / 100.0)
CAR_TT_SCALED = DefineVariable('CAR_TT_SCALED', CAR_TIME / 100)
TRAIN_EGDIS_SCALED = DefineVariable('TRAIN_EGDIS_SCALED', EGRESS_DIS_TRAIN/1000.0 )
TRAIN_ACCDIS_SCALED = DefineVariable('TRAIN_ACCDIS_SCALED', TRAIN_ACCESS_DIS/1000.0 )
TRAIN_COST_SCALED = DefineVariable('TRAIN_COST_SCALED', TRAIN_COST / 1000.0)
BUS_COST_SCALED = DefineVariable('BUS_COST_SCALED', BUS_COST / 100.0)


V1 = ASC_TRAIN + B_TIME * TRAIN_TT_SCALED +B_WORKING*WORKING+B_AGE30UNDER*AGE30UNDER+ B_COST * TRAIN_COST_SCALED+B_EGRESS_DIS_ACCESS*TRAIN_ACCDIS_SCALED+B_EGRESS_DIS_TRAIN*TRAIN_EGDIS_SCALED
V2 = ASC_BUS + B_TIME * BUS_TT_SCALED  + B_FEMALE * WOMEN + B_COST * BUS_COST_SCALED
V3 = ASC_CAR + B_TIME_CAR * CAR_TT_SCALED +B_AGE50OVER*AGE50OVER

# Associate utility functions with the numbering of alternatives
V = {200: V1,
     240: V2,
     100: V3}
#Associate the availability conditions with the alternatives
#CAR_AV_SP =  DefineVariable('CAR_AV_SP',CAR_AV  * (  SP   !=  0  ))
#TRAIN_AV_SP =  DefineVariable('TRAIN_AV_SP',TRAIN_AV  * (  SP   !=  0  ))

av = {200: TRAIN_AV,
      240: BUS_AV,
      100: CAR_AV}



#NLネストの設定
ROAD = MU_ROAD , [240,100]
Short = 1.0 , [200]
nests = ROAD,Short
# The choice model is a logit, with availability conditions
logprob = lognested(V,av,nests,TRANSPORTATION)
# Defines an itertor on the data
rowIterator('obsIter', '__dataFile__') 

# DEfine the likelihood function for the estimation
BIOGEME_OBJECT.ESTIMATE = Sum(logprob,'obsIter')

# All observations verifying the following expression will not be
# considered for estimation
# The modeler here has developed the model only for work trips.  
# Observations such that the dependent variable CHOICE is 0 are also removed.
exclude = ( TRANSPORTATION != 100 ) * (  TRANSPORTATION  !=  200  )  * ( TRANSPORTATION != 240 )  > 0

BIOGEME_OBJECT.EXCLUDE = exclude

# Statistics

nullLoglikelihood(av,'obsIter')
choiceSet = [200,240,100]
cteLoglikelihood(choiceSet,TRANSPORTATION,'obsIter')
availabilityStatistics(av,'obsIter')


BIOGEME_OBJECT.PARAMETERS['optimizationAlgorithm'] = "BIO"

BIOGEME_OBJECT.FORMULAS['Train utility'] = V1
BIOGEME_OBJECT.FORMULAS['BUS utility'] = V2
BIOGEME_OBJECT.FORMULAS['Car utility'] = V3